#include "stdafx.h"

#include "export.h"
#include "ff3se.h"
#include "resource.h"

ExportDialog exportDialog;

/*************************************/
ExportDialog::ExportDialog(void)
{
	isInit = false;
}

/*************************************/
//Brings up a save as common dialog box to get a new filename
void ExportDialog::browse(void)
{
	OPENFILENAME of;

	of.Flags = OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY;
	of.hInstance = hInst;
	of.hwndOwner = hDlg;
	of.lCustData = 0;
	of.lpfnHook = NULL;
	of.lpstrCustomFilter = NULL;
	if (fileType == 0)
		of.lpstrDefExt = ".bmp";
	else if (fileType == 1)
		of.lpstrDefExt = ".pcx";
	else if (fileType == 2)
		of.lpstrDefExt = ".png";
	else
		of.lpstrDefExt = ".bin";
	SendDlgItemMessage(hDlg, IDC_EDIT_EXPORT_FILENAME, WM_GETTEXT, 1024, (LPARAM)(LPCTSTR)filename);
	of.lpstrFile = filename;
	of.lpstrFileTitle = NULL;
	of.lpstrFilter = NULL;//"Supported images formats (*.png, *.bmp, *.pcx)\000*.png;*.bmp;*.pcx\000All files (*.*)\000*.*\000\000";
	of.lpstrInitialDir = NULL;
	of.lpstrTitle = dialogTitle;
	of.lpTemplateName = 0;
	of.lStructSize = sizeof(OPENFILENAME);
	of.nFileExtension = NULL;
	of.nFileOffset = NULL;
	of.nFilterIndex = NULL;
	of.nMaxCustFilter = 0;
	of.nMaxFile = 1024;
	of.nMaxFileTitle = 0;

	if (GetSaveFileName(&of) == 0)
		return;

	SendDlgItemMessage(hDlg, IDC_EDIT_EXPORT_FILENAME, WM_SETTEXT, 0, (LPARAM)(LPCTSTR)filename);

	changeFileType(filename);
}

/*************************************/
//Changes the extension of filename based on the filetype
void ExportDialog::changeFileExtension(void)
{
	SendDlgItemMessage(hDlg, IDC_EDIT_EXPORT_FILENAME, WM_GETTEXT, 1024, (LPARAM)(LPCTSTR)filename);
	int i;
	int len = strlen(filename);
	bool found = false;

	for (i = len - 1; (i > len - 5) && (!found); i--)
	{
		if (filename[i] == '.')
		{
			found = true;
			break;
		}
	}

	if (!found)
	{
		i = len;
		filename[i] = '.';
	}

	switch (fileType)
	{
	case 0:
		filename[i + 1] = 'b';
		filename[i + 2] = 'm';
		filename[i + 3] = 'p';
		filename[i + 4] = '\0';
		break;
	case 1:
		filename[i + 1] = 'p';
		filename[i + 2] = 'c';
		filename[i + 3] = 'x';
		filename[i + 4] = '\0';
		break;
	case 2:
		filename[i + 1] = 'p';
		filename[i + 2] = 'n';
		filename[i + 3] = 'g';
		filename[i + 4] = '\0';
		break;
	case 3:
		filename[i + 1] = 'b';
		filename[i + 2] = 'i';
		filename[i + 3] = 'n';
		filename[i + 4] = '\0';
		break;
	}

	SendDlgItemMessage(hDlg, IDC_EDIT_EXPORT_FILENAME, WM_SETTEXT, 0, (LPARAM)(LPCTSTR)filename);
}

/*************************************/
//Changes the filetype based on the filename passed in
void ExportDialog::changeFileType(char *file)
{
	int len = strlen(file);

	if (len < 4)
		return;

	if (file[len - 4] != '.')
		return;

	if ((tolower(file[len - 3]) == 'b') &&
		(tolower(file[len - 2]) == 'm') &&
		(tolower(file[len - 1]) == 'p'))
	{
		fileType = 0;
	}
	if ((tolower(file[len - 3]) == 'p') &&
		(tolower(file[len - 2]) == 'c') &&
		(tolower(file[len - 1]) == 'x'))
	{
		fileType = 1;
	}
	if ((tolower(file[len - 3]) == 'p') &&
		(tolower(file[len - 2]) == 'n') &&
		(tolower(file[len - 1]) == 'g'))
	{
		fileType = 2;
	}
	if ((tolower(file[len - 3]) == 'b') &&
		(tolower(file[len - 2]) == 'i') &&
		(tolower(file[len - 1]) == 'n'))
	{
		fileType = 3;
	}

	SendDlgItemMessage(hDlg, IDC_COMBO_EXPORT_FILETYPE, CB_SETCURSEL, fileType, 0);
}

/*************************************/
//Determines the file type to use by the selcetion in the combo box
void ExportDialog::getFileType(void)
{
	fileType = SendDlgItemMessage(hDlg, IDC_COMBO_EXPORT_FILETYPE, CB_GETCURSEL, 0, 0);
	changeFileExtension();
}

/*************************************/
void ExportDialog::init(HWND hdlg)
{
	RECT r;

	if (isInit)
		return;

	isInit = true;

	hDlg = hdlg;

	//Set the title
	if (dialogTitle != NULL)
		SetWindowText(hDlg, dialogTitle);
	else
		SetWindowText(hDlg, ff3seTitle);

	//Add file types to combo box
	SendDlgItemMessage(hDlg, IDC_COMBO_EXPORT_FILETYPE, CB_RESETCONTENT, 0, 0);
	SendDlgItemMessage(hDlg, IDC_COMBO_EXPORT_FILETYPE, CB_ADDSTRING, 0, (LPARAM)(LPCSTR)".BMP");
	SendDlgItemMessage(hDlg, IDC_COMBO_EXPORT_FILETYPE, CB_ADDSTRING, 0, (LPARAM)(LPCSTR)".PCX");
	SendDlgItemMessage(hDlg, IDC_COMBO_EXPORT_FILETYPE, CB_ADDSTRING, 0, (LPARAM)(LPCSTR)".PNG");
	if (m_allowBin)
	{
		SendDlgItemMessage(hDlg, IDC_COMBO_EXPORT_FILETYPE, CB_ADDSTRING, 0, (LPARAM)(LPCSTR)".BIN");
	}
	SendDlgItemMessage(hDlg, IDC_COMBO_EXPORT_FILETYPE, CB_SETCURSEL, fileType, 0);

	//Put filename in edit control
	SendDlgItemMessage(hDlg, IDC_EDIT_EXPORT_FILENAME, EM_SETLIMITTEXT, 1024, 0);
	SendDlgItemMessage(hDlg, IDC_EDIT_EXPORT_FILENAME, WM_SETTEXT, 0, (LPARAM)(LPCTSTR)filename);

	//Move window and display
	GetWindowRect(hWnd, &r);
	SetWindowPos(hDlg, HWND_NOTOPMOST, r.left + 150, r.top + 100, 0, 0, SWP_NOSIZE | SWP_SHOWWINDOW);
	UpdateWindow(hDlg);
}

/*************************************/
void ExportDialog::returnOK(HWND hdlg)
{
	EnableWindow(hWnd, true);

	EndDialog(hDlg, true);
}

/*************************************/
//Displays the Export Dialog to get a file name and file type to export the image to
//Returns false if cancel was selected
bool ExportDialog::show(char *title, char *name, int *type, bool allowBin)
{
	EnableWindow(hWnd, false);

	strcpy(dialogTitle, title);
	strcpy(filename, name);
	fileType = *type;
	m_allowBin = allowBin;

	returnValue = (bool)DialogBox(hInst, MAKEINTRESOURCE(IDD_EXPORT), hWnd, ::exportProc);

	if (returnValue)
	{
		strcpy(name, filename);
		*type = fileType;
	}

	isInit = false;

	return returnValue;
}

/*************************************/
BOOL ExportDialog::exportProc(HWND hdlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_ACTIVATE:
		init(hdlg);
		break;
	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDC_EXPORT_OK:
			returnOK(hdlg);
			break;
		case IDC_EXPORT_CANCEL:
			EnableWindow(hWnd, true);
			EndDialog(hDlg, false);
			break;
		case IDC_BUTTON_EXPORT_BROWSE:
			browse();
			break;
		case IDC_COMBO_EXPORT_FILETYPE:
			if (HIWORD(wParam) == CBN_SELCHANGE)
			{
				getFileType();
			}
			break;
		}
		break;
	}

	return false;
}

/*************************************/
BOOL CALLBACK exportProc(HWND hdlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	return exportDialog.exportProc(hdlg, message, wParam, lParam);
}
