#include "stdafx.h"

#include "ff3se.h"
#include "tab3.h"
#include "resource.h"
#include "portrait.h"
#include "options.h"
#include "export.h"

Tab3 tab3;

char *portNames[19] = { "Ghost","Banon","Leo","Imp","Vicks & Wedge","Umaro","Gogo","Gau","Mog","Setzer",
					   "Relm","Strago","Celes","Sabin","Edgar","Shadow","Cyan","Locke","Terra" };

/*************************************/
//Destructor for Tab3
Tab3::~Tab3(void)
{
	DeleteDC(portHdc);
	DeleteDC(editHdc);

	DeleteObject(portBMP);
	DeleteObject(editBMP);
}

/*************************************/
//Adds the list of character names to the combo box
void Tab3::addNames(void)
{
	int i;

	SendDlgItemMessage(hTab, IDC_COMBO_PORTRAIT, CB_RESETCONTENT, 0, 0);

	for (i = 18; i >= 0; i--)
	{
		SendDlgItemMessage(hTab, IDC_COMBO_PORTRAIT, CB_ADDSTRING, 0,
			(WPARAM)(LPCSTR)portNames[i]);
	}

	SendDlgItemMessage(hTab, IDC_COMBO_PORTRAIT, CB_SETCURSEL, 0, 0);

	portSelected = 0;
}

/*************************************/
//Handles the action of the brush button
//Sets the pixel x,y of the portrait palette entry selected
void Tab3::brushButton(int x, int y)
{
	m_changed = true;

	if ((x >= 40) || (y >= 40))
		return;

	portrait[portSelected].setPixel(x, y, palEntrySelected);

	invalidatePics();
}

/*************************************/
//Changes which palette entry is selected
void Tab3::changePalEntry(BYTE num)
{
	palEntrySelected = num;
	sliderEnable();
	invalidateTabControl(IDC_PIC_PORTRAIT_PALETTE);
}

/*************************************/
//Changes the palette used by the Portrait to the palette selected in the combo box
void Tab3::changePalette(void)
{
	int palSelected;

	m_changed = true;

	palSelected = SendDlgItemMessage(hTab, IDC_COMBO_PORTRAIT_PALETTE, CB_GETCURSEL, 0, 0);
	portrait[portSelected].changePalette(palSelected);

	invalidatePics();
}

/*************************************/
//Changes which color is transparent
//Swaps the currently selected color with color 0
void Tab3::changeTransparency(void)
{
	m_changed = true;

	portrait[portSelected].setTransparency(palEntrySelected);

	changePalEntry(0);

	invalidatePics();
}

/*************************************/
//Copies another Portrait's image
void Tab3::copyPortrait(void)
{
	int i;

	m_changed = true;

	hDlg = CreateDialog(hInst, MAKEINTRESOURCE(IDD_COPY_IMAGE), hTab, ::copyPortraitProc);

	SendDlgItemMessage(hDlg, IDC_COPY_LIST, LB_RESETCONTENT, 0, 0);
	for (i = 0; i < 19; i++)
	{
		SendDlgItemMessage(hDlg, IDC_COPY_LIST, LB_ADDSTRING, 0,
			(LPARAM)(LPCTSTR)portNames[18 - i]);
	}

	SetWindowText(hDlg, "Copy portrait image");
	BringWindowToTop(hDlg);
	ShowWindow(hDlg, SW_NORMAL);
	SetFocus(hDlg);
	EnableWindow(hWnd, false);
}

/*************************************/
//Clears the Portrait
void Tab3::clearImage(void)
{
	if (MessageBox(NULL, "This will erase the current portrait", ff3seTitle, MB_OKCANCEL | MB_TASKMODAL) != IDOK)
		return;

	m_changed = true;

	portrait[portSelected].clear();

	invalidatePics();
}

/*************************************/
//Creates the tab
void Tab3::create(void)
{
	hTab = CreateDialog(hInst, MAKEINTRESOURCE(IDD_TAB3), ::hMainTab, tab3Proc);
	SetWindowPos(hTab, HWND_TOP, 5, 30, 680, 480, 0);
}

/*************************************/
//Updates HDCs for the portrait and editing pic with the new bitmap images
void Tab3::createHDCs(void)
{
	int xScroll, yScroll;

	xScroll = SendDlgItemMessage(hTab, IDC_HSCROLL_PORTRAIT_EDIT, SBM_GETPOS, 0, 0);
	yScroll = SendDlgItemMessage(hTab, IDC_VSCROLL_PORTRAIT_EDIT, SBM_GETPOS, 0, 0);

	portrait[portSelected].pic.drawToDC(portHdc, &backBmp, 40, 40);

	portrait[portSelected].pic.drawToLargeDC(
		magnification, editHdc, xScroll, yScroll, &back2Bmp, 256, 256);
}

/*************************************/
//Displays the preview of the file selected in the import dialog box
void Tab3::displayPreview(HWND hdlg)
{
	char filename[200];
	HDC picHdc, bmpHdc;
	HWND hPic, hText;
	Bitmap bmp;
	BYTE result;
	HBITMAP tempBMP;
	int size;

	size = CommDlg_OpenSave_GetSpec(GetParent(hdlg), filename, 200);
	if (size == 1)		//Exit if no file selected
	{
		SendDlgItemMessage(hdlg, IDC_TEXT_COLORS, WM_SETTEXT, (WPARAM)0, (LPARAM)(LPCTSTR)"");
		SendDlgItemMessage(hdlg, IDC_TEXT_SIZE, WM_SETTEXT, (WPARAM)0, (LPARAM)(LPCTSTR)"");
		return;
	}
	CommDlg_OpenSave_GetFilePath(GetParent(hdlg), filename, 200);
	hPic = GetDlgItem(hdlg, IDC_MONSPREVIEW);
	picHdc = GetDC(hPic);							//Create an HDC for the preview picture control

	result = bmp.loadFile(filename, 16, 40, 40);
	if (result != 0)
	{
		backBmp.drawToDC(picHdc);

		//Set text message to display reason image can't be previewed
		SendDlgItemMessage(hdlg, IDC_TEXT_COLORS, WM_SETTEXT, (WPARAM)0, (LPARAM)(LPCTSTR)"");
		if (result == 1)
		{
			SendDlgItemMessage(hdlg, IDC_TEXT_SIZE, WM_SETTEXT, (WPARAM)0,
				(LPARAM)(LPCTSTR)"Image has over 16 colors");
		}
		else if (result == 2)
		{
			SendDlgItemMessage(hdlg, IDC_TEXT_SIZE, WM_SETTEXT, (WPARAM)0,
				(LPARAM)(LPCTSTR)"Image is larger than 40x40");
		}
		else if (result == 3)
		{
			SendDlgItemMessage(hdlg, IDC_TEXT_SIZE, WM_SETTEXT, (WPARAM)0,
				(LPARAM)(LPCTSTR)"Error opening image");
		}
		else if (result == 4)
		{
			SendDlgItemMessage(hdlg, IDC_TEXT_SIZE, WM_SETTEXT, (WPARAM)0,
				(LPARAM)(LPCTSTR)"Not a valid image file");
		}
	}
	else
	{
		bmpHdc = CreateCompatibleDC(screenHdc);
		tempBMP = CreateCompatibleBitmap(screenHdc, 128, 128);
		SelectObject(bmpHdc, tempBMP);
		bmp.drawToDC(bmpHdc, &backBmp, 128, 128);

		BitBlt(picHdc, 0, 0, 128, 128, bmpHdc, 0, 0, SRCCOPY);

		hText = GetDlgItem(hdlg, IDC_TEXT_COLORS);

		SendDlgItemMessage(hdlg, IDC_TEXT_COLORS, WM_SETTEXT, (WPARAM)0, (LPARAM)(LPCTSTR)"");
		SendDlgItemMessage(hdlg, IDC_TEXT_SIZE, WM_SETTEXT, (WPARAM)0, (LPARAM)(LPCTSTR)"");
	}


	DeleteDC(bmpHdc);
	DeleteDC(picHdc);
	DeleteObject(tempBMP);
}

/*************************************/
//Redraws the paint buttons
void Tab3::drawPaintButtons(WPARAM wParam, LPARAM lParam)
{
	HBITMAP BMP;
	HDC bufferHdc;

	switch (((LPDRAWITEMSTRUCT)lParam)->CtlID)
	{
	case IDC_BUTTON_PORTRAIT_BRUSH:
		if (buttonSelected == 0)
			BMP = (HBITMAP)LoadImage(hInst, MAKEINTRESOURCE(IDB_BRUSH2), IMAGE_BITMAP, 42, 42, LR_DEFAULTCOLOR);
		else
			BMP = (HBITMAP)LoadImage(hInst, MAKEINTRESOURCE(IDB_BRUSH), IMAGE_BITMAP, 42, 42, LR_DEFAULTCOLOR);
		bufferHdc = CreateCompatibleDC(screenHdc);
		SelectObject(bufferHdc, BMP);
		BitBlt(((LPDRAWITEMSTRUCT)lParam)->hDC, 0, 0, 42, 42, bufferHdc, 0, 0, SRCCOPY);
		break;
	case IDC_BUTTON_PORTRAIT_ERASE:
		if (buttonSelected == 1)
			BMP = (HBITMAP)LoadImage(hInst, MAKEINTRESOURCE(IDB_ERASE2), IMAGE_BITMAP, 42, 42, LR_DEFAULTCOLOR);
		else
			BMP = (HBITMAP)LoadImage(hInst, MAKEINTRESOURCE(IDB_ERASE), IMAGE_BITMAP, 42, 42, LR_DEFAULTCOLOR);
		bufferHdc = CreateCompatibleDC(screenHdc);
		SelectObject(bufferHdc, BMP);
		BitBlt(((LPDRAWITEMSTRUCT)lParam)->hDC, 0, 0, 42, 42, bufferHdc, 0, 0, SRCCOPY);
		break;
	case IDC_BUTTON_PORTRAIT_GRAB:
		if (buttonSelected == 2)
			BMP = (HBITMAP)LoadImage(hInst, MAKEINTRESOURCE(IDB_GRAB2), IMAGE_BITMAP, 42, 42, LR_DEFAULTCOLOR);
		else
			BMP = (HBITMAP)LoadImage(hInst, MAKEINTRESOURCE(IDB_GRAB), IMAGE_BITMAP, 42, 42, LR_DEFAULTCOLOR);
		bufferHdc = CreateCompatibleDC(screenHdc);
		SelectObject(bufferHdc, BMP);
		BitBlt(((LPDRAWITEMSTRUCT)lParam)->hDC, 0, 0, 42, 42, bufferHdc, 0, 0, SRCCOPY);
		break;
	}
	DeleteDC(bufferHdc);
	DeleteObject(BMP);
}

/*************************************/
void Tab3::drawTab(void)
{
	HDC picHdc;
	HDC bufferHdc;
	HWND hPic;
	HBITMAP bufferBMP;
	PAINTSTRUCT ps;
	RECT r, r2;
	int x, y;
	int i, j;
	long color;
	int g;
	int maxX, maxY;

	if (!hTab)		//Exit if attempting to draw before the tab is fully initiliazed
		return;

	createHDCs();

	GetUpdateRect(hTab, &r, false);

	ps.fErase = false;
	ps.rcPaint = r;
	GetWindowRect(hTab, &r2);

	r.bottom += r2.top;		//Convert update rect to screen coordinates
	r.left += r2.left;
	r.right += r2.left;
	r.top += r2.top;

	//Draw portrait pic
	hPic = GetDlgItem(hTab, IDC_PIC_PORTRAIT);

	GetWindowRect(hPic, &r2);

	if (overlapRect(r, r2))			//Check if portrait pic needs to be updated
	{
		picHdc = GetDC(hPic);
		ps.hdc = picHdc;

		BeginPaint(hTab, &ps);

		BitBlt(picHdc, 0, 0, 40, 40, portHdc, 0, 0, SRCCOPY);

		EndPaint(hTab, &ps);
		ReleaseDC(hPic, picHdc);
	}

	//Draw zoomed in portrait editing pic
	hPic = GetDlgItem(hTab, IDC_PIC_EDITPORTRAIT);

	GetWindowRect(hPic, &r2);

	if (overlapRect(r, r2))			//Check if edit portrait pic needs to be updated
	{
		picHdc = GetDC(hPic);
		ps.hdc = picHdc;

		BeginPaint(hTab, &ps);

		maxX = 40 * magnification;
		if (maxX > 256)
			maxX = 256;
		maxY = maxX;

		g = options.drawGrid(magnification);
		if (g == 1)		//Show pixel grid
		{
			for (i = 0; i <= maxY; i += magnification)
			{
				for (j = 0; j <= maxX; j++)
				{
					SetPixel(editHdc, i, j, 0x888888);
					SetPixel(editHdc, j, i, 0x888888);
				}
			}
		}
		if (g == 2)		//Show tile grid
		{
			int xoff, yoff;
			int xScroll, yScroll;

			xScroll = SendDlgItemMessage(hTab, IDC_HSCROLL_MONSTER_EDIT, SBM_GETPOS, 0, 0);
			yScroll = SendDlgItemMessage(hTab, IDC_VSCROLL_MONSTER_EDIT, SBM_GETPOS, 0, 0);

			xoff = (xScroll % 8) * magnification;
			yoff = (yScroll % 8) * magnification;

			for (i = 0; i <= maxY; i += magnification * 8)
			{
				for (j = 0; j <= maxX; j++)
				{
					x = i - xoff;
					if (x < 0)
						x += (256 - (256 % (magnification * 8))) + (magnification * 8);
					y = j;
					SetPixel(editHdc, x, y, 0x888888);
					y = i - yoff;
					if (y < 0)
						y += (256 - (256 % (magnification * 8))) + (magnification * 8);
					x = j;
					SetPixel(editHdc, x, y, 0x888888);
				}
			}
		}

		BitBlt(picHdc, 0, 0, 256, 256, editHdc, 0, 0, SRCCOPY);

		EndPaint(hTab, &ps);
		ReleaseDC(hPic, picHdc);
	}

	//Draw palette pic				
	hPic = GetDlgItem(hTab, IDC_PIC_PORTRAIT_PALETTE);

	GetWindowRect(hPic, &r2);

	if (overlapRect(r, r2))			//Check if palette pic needs to be updated
	{
		picHdc = GetDC(hPic);
		ps.hdc = picHdc;

		//Create a buffer to draw the palette on
		bufferHdc = CreateCompatibleDC(screenHdc);
		bufferBMP = CreateCompatibleBitmap(screenHdc, 288, 30);
		SelectObject(bufferHdc, bufferBMP);

		//Draw the palette with a border around each color
		for (i = 0; i < 16; i++)
		{
			color = RGBtoCOLORREF(portraitPals[portrait[portSelected].getPalNum()][i]);
			for (y = 0; y < 30; y++)
			{
				if (palEntrySelected == i)
					SetPixel(bufferHdc, i * 18, y, 0xFFFFFF);
				else
					SetPixel(bufferHdc, i * 18, y, 0);
				for (x = 1; x < 17; x++)
				{
					if ((y == 0) || (y == 29))
						if (palEntrySelected == i)
							SetPixel(bufferHdc, x + i * 18, y, 0xFFFFFF);
						else
							SetPixel(bufferHdc, x + i * 18, y, 0);
					else
						SetPixel(bufferHdc, x + i * 18, y, color);
				}
				if (palEntrySelected == i)
					SetPixel(bufferHdc, x + i * 18, y, 0xFFFFFF);
				else
					SetPixel(bufferHdc, x + i * 18, y, 0);
			}
		}

		//Draw buffer to screen
		BeginPaint(hTab, &ps);

		BitBlt(picHdc, 0, 0, 288, 30, bufferHdc, 0, 0, SRCCOPY);

		EndPaint(hTab, &ps);
		ReleaseDC(hPic, picHdc);
		DeleteDC(bufferHdc);
		DeleteObject(bufferBMP);
	}

	updateControls();
	
}

/*************************************/
//Handles the action of the erase button
//Sets the pixel x,y of the portrait to color 0 (transparent)
void Tab3::eraseButton(int x, int y)
{
	m_changed = true;

	if ((x >= 40) || (y >= 40))
		return;

	portrait[portSelected].setPixel(x, y, 0);

	invalidatePics();
}

/*************************************/
void Tab3::exportImage(void)
{
	char filename[1024];
	int fileType;

	sprintf(filename, "%s.png", portNames[18 - portSelected]);
	fileType = 2;

	if (exportDialog.show("Export portrait", filename, &fileType, true))
	{
		switch (fileType)
		{
		case 0:
			portrait[portSelected].pic.saveBMP(filename, 40, 40);
			break;
		case 1:
			portrait[portSelected].pic.savePCX(filename, 40, 40);
			break;
		case 2:
			portrait[portSelected].pic.savePNG(filename, 40, 40);
			break;
		case 3:
			auto handle = _open(filename, _O_WRONLY | _O_BINARY | _O_TRUNC | _O_CREAT, _S_IREAD | _S_IWRITE);
			if (handle == -1)
			{
				MessageBox(NULL, "Error saving portrait data", ff3seTitle, MB_OK | MB_ICONWARNING | MB_TASKMODAL);
				return;
			}
			portrait[portSelected].save(handle);
			_close(handle);

			size_t len = strnlen(filename, 1024);
			if (len > 4)
			{
				filename[len - 4] = '.';
				filename[len - 3] = 'p';
				filename[len - 2] = 'a';
				filename[len - 1] = 'l';
			}
			else
			{
				strcat(filename, ".pal");
			}

			handle = _open(filename, _O_WRONLY | _O_BINARY | _O_TRUNC | _O_CREAT, _S_IREAD | _S_IWRITE);
			if (handle == -1)
			{
				MessageBox(NULL, "Error saving portrait data", ff3seTitle, MB_OK | MB_ICONWARNING | MB_TASKMODAL);
				return;
			}

			int paletteNumber = portrait[portSelected].getPalNum();

			BYTE c, d;
			BYTE temp[32 * 19];
			BYTE *ptr;
			RGBTRIPLE color;

			ptr = temp;
			for (int j = 0; j < 16; j++)
			{
				color = portraitPals[paletteNumber][j];
				getIntPal(color, &c, &d);
				*(ptr++) = c;
				*(ptr++) = d;
			}
			_write(handle, &temp, 32);
			_close(handle);
		}
	}
}

/*************************************/
void Tab3::flipImage(int flip)
{
	portrait[portSelected].flipImage(flip);
	invalidatePics();
}

/*************************************/
//Handles the scroll bars for the editing pic
void Tab3::handleMagScroll(int val, HWND hScroll, int pos)
{
	switch (val)
	{
	case SB_LINELEFT:
		pos = GetScrollPos(hScroll, SB_CTL) - 1;
		SetScrollPos(hScroll, SB_CTL, pos, true);
		break;
	case SB_LINERIGHT:
		pos = GetScrollPos(hScroll, SB_CTL) + 1;
		SetScrollPos(hScroll, SB_CTL, pos, true);
		break;
	case SB_PAGELEFT:
		pos = GetScrollPos(hScroll, SB_CTL) - 10;
		SetScrollPos(hScroll, SB_CTL, pos, true);
		break;
	case SB_PAGERIGHT:
		pos = GetScrollPos(hScroll, SB_CTL) + 10;
		SetScrollPos(hScroll, SB_CTL, pos, true);
		break;
	case SB_THUMBTRACK:
		SetScrollPos(hScroll, SB_CTL, pos, true);
	}

}

/*************************************/
//Determines where the mouse was clicked and handles any operations that need to be done
//Changes the palette selction if the palette bar is clicked on.
void Tab3::handleMouseClick(int x, int y)
{
	int tabx, taby;
	HWND hPal, hEdit;
	BYTE c;

	RECT r, r2;

	GetWindowRect(hTab, &r);

	tabx = r.left;
	taby = r.top;

	x += tabx;
	y += taby;

	//Check if the mouse was clicked on the palette bar

	hPal = GetDlgItem(hTab, IDC_PIC_PORTRAIT_PALETTE);
	GetWindowRect(hPal, &r);

	if (inRect(x, y, r))
	{
		changePalEntry((x - r.left) / 18);
	}

	//Check if the mouse was clicked on the editing pic and fill was selected

	hEdit = GetDlgItem(hTab, IDC_PIC_EDITPORTRAIT);
	GetWindowRect(hEdit, &r);

	if (inRect(x, y, r))
	{
		GetWindowRect(hEdit, &r2);

		//Find what pixel was clicked on
		y -= r2.top;
		x -= r2.left;

		x /= magnification;
		y /= magnification;

		x += SendDlgItemMessage(hTab, IDC_HSCROLL_PORTRAIT_EDIT, SBM_GETPOS, 0, 0);
		y += SendDlgItemMessage(hTab, IDC_VSCROLL_PORTRAIT_EDIT, SBM_GETPOS, 0, 0);

		if (buttonSelected == 0)
		{
			brushButton(x, y);
		}
		if (buttonSelected == 1)
		{
			eraseButton(x, y);
		}
		if (buttonSelected == 2)
		{
			c = portrait[portSelected].getPixel(x, y);

			changePalEntry(c);
		}
	}

}

/*************************************/
//Determines where the mouse is currently at
//Down is set if the left mouse button is pressed
//Handles updating the portrait image, if the editing pic is clicked on
void Tab3::handleMouseMove(int x, int y, bool down)
{

	if (!down)			//Don't edit the pic unless the left mouse button is pressed
		return;

	int tabx, taby;
	HWND hEdit;
	BYTE c;
	RECT r, r2;

	GetWindowRect(hTab, &r);

	tabx = r.left;
	taby = r.top;

	x += tabx;
	y += taby;

	//Check if the mouse was clicked on the editing pic

	hEdit = GetDlgItem(hTab, IDC_PIC_EDITPORTRAIT);
	GetWindowRect(hEdit, &r);

	if (inRect(x, y, r))
	{
		GetWindowRect(hEdit, &r2);

		//Find what pixel was clicked on
		y -= r2.top;
		x -= r2.left;

		x /= magnification;
		y /= magnification;

		x += SendDlgItemMessage(hTab, IDC_HSCROLL_PORTRAIT_EDIT, SBM_GETPOS, 0, 0);
		y += SendDlgItemMessage(hTab, IDC_VSCROLL_PORTRAIT_EDIT, SBM_GETPOS, 0, 0);

		if (buttonSelected == 0)
		{
			brushButton(x, y);
		}
		if (buttonSelected == 1)
		{
			eraseButton(x, y);
		}
		if (buttonSelected == 2)
		{
			c = portrait[portSelected].getPixel(x, y);

			changePalEntry(c);
		}
	}
}

/*************************************/
//Handles functions for the palette sliders
void Tab3::handleSliders(WPARAM wParam, LPARAM lParam)
{
	HWND hSlider;
	BYTE val;
	int i;

	m_changed = true;

	hSlider = GetDlgItem(hTab, IDC_SLIDER_PORTRAIT_RED);

	if ((HWND)lParam == hSlider)
	{
		val = (BYTE)SendMessage((HWND)lParam, TBM_GETPOS, 0, 0);
		portraitPals[portrait[portSelected].getPalNum()][palEntrySelected].rgbtRed = val * 8;
		for (i = 0; i < 19; i++)
			portrait[i].updatePalette();
		invalidatePics();
	}

	hSlider = GetDlgItem(hTab, IDC_SLIDER_PORTRAIT_GREEN);

	if ((HWND)lParam == hSlider)
	{
		val = (BYTE)SendMessage((HWND)lParam, TBM_GETPOS, 0, 0);
		portraitPals[portrait[portSelected].getPalNum()][palEntrySelected].rgbtGreen = val * 8;
		for (i = 0; i < 19; i++)
			portrait[i].updatePalette();
		invalidatePics();
	}

	hSlider = GetDlgItem(hTab, IDC_SLIDER_PORTRAIT_BLUE);

	if ((HWND)lParam == hSlider)
	{
		val = (BYTE)SendMessage((HWND)lParam, TBM_GETPOS, 0, 0);
		portraitPals[portrait[portSelected].getPalNum()][palEntrySelected].rgbtBlue = val * 8;
		for (i = 0; i < 19; i++)
			portrait[i].updatePalette();
		invalidatePics();
	}

}

/*************************************/
void Tab3::import(void)
{
	OPENFILENAME of;

	char filename[256];
	char *title;

	title = (char *)malloc(50);
	strcpy(title, "Import Portrait");

	ZeroMemory(filename, 256);

	of.Flags = OFN_ENABLETEMPLATE | OFN_EXPLORER | OFN_HIDEREADONLY
		| OFN_FILEMUSTEXIST | OFN_ENABLEHOOK;
	of.hInstance = hInst;
	of.hwndOwner = hTab;
	of.lCustData = 0;
	of.lpfnHook = ::tab3ImportProc;
	of.lpstrCustomFilter = NULL;
	of.lpstrDefExt = NULL;
	of.lpstrFile = filename;
	of.lpstrFileTitle = NULL;
	of.lpstrFilter = "Supported images formats (*.png, *.bmp, *.pcx)\000*.png;*.bmp;*.pcx\000*.bin+*.pal\000*.bin\000All files (*.*)\000*.*\000\000";
	of.lpstrInitialDir = NULL;
	of.lpstrTitle = title;
	of.lpTemplateName = MAKEINTRESOURCE(IDD_IMPORT_MONS);
	of.lStructSize = sizeof(OPENFILENAME);
	of.nFileExtension = 0;
	of.nFileOffset = 0;
	of.nFilterIndex = 0;
	of.nMaxCustFilter = 0;
	of.nMaxFile = 256;
	of.nMaxFileTitle = 256;

	GetOpenFileName(&of);

	m_changed = true;
}

//*************************************/
void Tab3::initTab(void)
{
	int i;
	char temp[20];
	char temp2[20];

	portBMP = CreateCompatibleBitmap(screenHdc, 40, 40);
	editBMP = CreateCompatibleBitmap(screenHdc, 256, 256);

	portHdc = CreateCompatibleDC(screenHdc);
	editHdc = CreateCompatibleDC(screenHdc);

	SelectObject(portHdc, portBMP);
	SelectObject(editHdc, editBMP);

	addNames();

	changePalEntry(0);

	SendDlgItemMessage(hTab, IDC_COMBO_PORTRAIT_PALETTE, CB_RESETCONTENT, 0, 0);
	for (i = 0; i < 19; i++)
	{
		strcpy(temp, "Palette #");
		strcat(temp, _itoa(i, temp2, 10));
		SendDlgItemMessage(hTab, IDC_COMBO_PORTRAIT_PALETTE, CB_ADDSTRING, 0,
			(LPARAM)(LPCTSTR)temp);
	}

	//Set ranges for color sliders
	SendDlgItemMessage(hTab, IDC_SLIDER_PORTRAIT_RED, TBM_SETRANGE, true, MAKELONG(0, 32));
	SendDlgItemMessage(hTab, IDC_SLIDER_PORTRAIT_GREEN, TBM_SETRANGE, true, MAKELONG(0, 32));
	SendDlgItemMessage(hTab, IDC_SLIDER_PORTRAIT_BLUE, TBM_SETRANGE, true, MAKELONG(0, 32));

	magnification = options.getTab3Mag();

	SendDlgItemMessage(hTab, IDC_UPDOWN_PORTRAIT_MAGNIFICATION, UDM_SETRANGE, 0, (LPARAM)MAKELONG((short)20, (short)2));
	SendDlgItemMessage(hTab, IDC_UPDOWN_PORTRAIT_MAGNIFICATION, UDM_SETPOS, 0, (LPARAM)MAKELONG((short)magnification, 0));

	portSelected = 0;
}

/*************************************/
//Invalidates the portrait pic,the editing pic, and the palette bar
//Calls drawTab to immediately update and redraw the tab
void Tab3::invalidatePics(void)
{
	invalidateTabControl(IDC_PIC_PORTRAIT);
	invalidateTabControl(IDC_PIC_EDITPORTRAIT);
	invalidateTabControl(IDC_PIC_PORTRAIT_PALETTE);

	drawTab();
}

/*************************************/
//handles the selection of a new character's portrait to edit
void Tab3::newSelection(void)
{
	int i;

	portSelected = (BYTE)SendDlgItemMessage(hTab, IDC_COMBO_PORTRAIT, CB_GETCURSEL, 0, 0);

	i = portrait[portSelected].getPalNum();
	SendDlgItemMessage(hTab, IDC_COMBO_PORTRAIT_PALETTE, CB_SETCURSEL, i, 0);

	invalidatePics();
}

/*************************************/
//Saves the changed portraits to ROM
bool Tab3::save_internal(char *filename)
{
	return savePortraits(filename);
}

/*************************************/
//Selected the nth paint button
void Tab3::selectButton(BYTE n)
{
	buttonSelected = n;
	invalidateTab();
}

/*************************************/
//If the mouse is over the editing pic, changes the cursor to the appropriate cursor
void Tab3::setCursor(void)
{
	POINT mousePos;
	HWND hEdit;
	RECT r;

	GetCursorPos(&mousePos);

	hEdit = GetDlgItem(hTab, IDC_PIC_EDITPORTRAIT);
	GetWindowRect(hEdit, &r);

	if (inRect(mousePos.x, mousePos.y, r))
	{
		switch (buttonSelected)
		{
		case 0:
			SetCursor(brushCursor);
			break;
		case 1:
			SetCursor(eraseCursor);
			break;
		case 2:
			SetCursor(grabCursor);
			break;
		}
	}
	else
		SetCursor(LoadCursor(NULL, IDC_ARROW));
}

/*************************************/
//If the selected palette entry is 0 disables the palette sliders, otherwise enables them
void Tab3::sliderEnable(void)
{
	HWND hSlider;

	if (palEntrySelected == 0)
	{
		hSlider = GetDlgItem(hTab, IDC_SLIDER_PORTRAIT_RED);
		EnableWindow(hSlider, false);
		hSlider = GetDlgItem(hTab, IDC_SLIDER_PORTRAIT_GREEN);
		EnableWindow(hSlider, false);
		hSlider = GetDlgItem(hTab, IDC_SLIDER_PORTRAIT_BLUE);
		EnableWindow(hSlider, false);
		hSlider = GetDlgItem(hTab, IDC_CHECK_PORTRAIT_TRANSPARENT);
		EnableWindow(hSlider, false);
		SendMessage(hSlider, BM_SETCHECK, (WPARAM)BST_CHECKED, 0);
	}
	else
	{
		hSlider = GetDlgItem(hTab, IDC_SLIDER_PORTRAIT_RED);
		EnableWindow(hSlider, true);
		hSlider = GetDlgItem(hTab, IDC_SLIDER_PORTRAIT_GREEN);
		EnableWindow(hSlider, true);
		hSlider = GetDlgItem(hTab, IDC_SLIDER_PORTRAIT_BLUE);
		EnableWindow(hSlider, true);
		hSlider = GetDlgItem(hTab, IDC_CHECK_PORTRAIT_TRANSPARENT);
		EnableWindow(hSlider, true);
		SendMessage(hSlider, BM_SETCHECK, (WPARAM)BST_UNCHECKED, 0);
	}
}

/*************************************/
//Updates all the combo boxes, sliders, etc.
void Tab3::updateControls(void)
{
	char temp[10];
	char mag[20];
	int maxScroll;

	int i;

	SendDlgItemMessage(hTab, IDC_SLIDER_PORTRAIT_RED, TBM_SETPOS, true,
		portraitPals[portrait[portSelected].getPalNum()][palEntrySelected].rgbtRed / 8);
	SendDlgItemMessage(hTab, IDC_SLIDER_PORTRAIT_GREEN, TBM_SETPOS, true,
		portraitPals[portrait[portSelected].getPalNum()][palEntrySelected].rgbtGreen / 8);
	SendDlgItemMessage(hTab, IDC_SLIDER_PORTRAIT_BLUE, TBM_SETPOS, true,
		portraitPals[portrait[portSelected].getPalNum()][palEntrySelected].rgbtBlue / 8);

	SendDlgItemMessage(hTab, IDC_TEXT_PORTRAIT_REDVAL, WM_SETTEXT, (WPARAM)0,
		(LPARAM)(LPCTSTR)_itoa(portraitPals[portrait[portSelected].getPalNum()][palEntrySelected].rgbtRed / 8,
			temp, 10));
	SendDlgItemMessage(hTab, IDC_TEXT_PORTRAIT_GREENVAL, WM_SETTEXT, (WPARAM)0,
		(LPARAM)(LPCTSTR)_itoa(portraitPals[portrait[portSelected].getPalNum()][palEntrySelected].rgbtGreen / 8,
			temp, 10));
	SendDlgItemMessage(hTab, IDC_TEXT_PORTRAIT_BLUEVAL, WM_SETTEXT, (WPARAM)0,
		(LPARAM)(LPCTSTR)_itoa(portraitPals[portrait[portSelected].getPalNum()][palEntrySelected].rgbtBlue / 8,
			temp, 10));

	i = portrait[portSelected].getPalNum();
	SendDlgItemMessage(hTab, IDC_COMBO_PORTRAIT_PALETTE, CB_SETCURSEL, i, 0);

	magnification = (BYTE)SendDlgItemMessage(hTab, IDC_UPDOWN_PORTRAIT_MAGNIFICATION, UDM_GETPOS, 0, 0);

	if (magnification == 0)
		magnification = 1;

	options.setTab3Mag(magnification);

	sprintf(mag, "magnification: x%d", magnification);
	SendDlgItemMessage(hTab, IDC_TEXT_PORTRAIT_MAGNIFICATION, WM_SETTEXT, 0, (LPARAM)(LPCTSTR)(mag));

	maxScroll = 40 - (256 / magnification);

	if (maxScroll < 0)
	{
		maxScroll = 0;
	}

	HWND hScroll = GetDlgItem(hTab, IDC_HSCROLL_PORTRAIT_EDIT);
	SetScrollRange(hScroll, SB_CTL, 0, maxScroll, true);
	if (maxScroll > 0)
		ShowWindow(hScroll, true);
	else
		ShowWindow(hScroll, false);

	hScroll = GetDlgItem(hTab, IDC_VSCROLL_PORTRAIT_EDIT);
	SetScrollRange(hScroll, SB_CTL, 0, maxScroll, true);
	if (maxScroll > 0)
		ShowWindow(hScroll, true);
	else
		ShowWindow(hScroll, false);
}

/*************************************/
bool Tab3::copyImageProc(HWND hdlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	if (message == WM_COMMAND)
	{
		switch (LOWORD(wParam))
		{
		case IDC_COPY_LIST:
			if (HIWORD(wParam) != LBN_DBLCLK)
				break;
		case IDC_COPY_OK:
			HWND hList;
			int image;

			hList = GetDlgItem(hdlg, IDC_COPY_LIST);
			image = SendDlgItemMessage(hdlg, IDC_COPY_LIST, LB_GETCURSEL, 0, 0);

			portrait[portSelected].copy(image);

			invalidatePics();

			BringWindowToTop(hWnd);
			EnableWindow(hWnd, true);
			DestroyWindow(hdlg);
			return true;
			break;
		case IDC_COPY_CANCEL:
			BringWindowToTop(hWnd);
			EnableWindow(hWnd, true);
			DestroyWindow(hdlg);
			hDlg = NULL;
			return true;
			break;
		}
	}
	return false;
}

/*************************************/
UINT Tab3::importProc(HWND hdlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	if (message == WM_NOTIFY)
	{
		switch (((LPOFNOTIFY)lParam)->hdr.code)
		{
		case CDN_SELCHANGE:
			displayPreview(hdlg);
			return 1;
			break;
		case CDN_FILEOK:
			const char* filename = ((LPOFNOTIFY)lParam)->lpOFN->lpstrFile;
			int len = strlen(filename);

			if (len > 4 && (tolower(filename[len - 4]) == '.') &&
				(tolower(filename[len - 3]) == 'b') &&
				(tolower(filename[len - 2]) == 'i') &&
				(tolower(filename[len - 1]) == 'n'))
			{
				char paletteFilename[MAX_PATH];
				len = min(len, MAX_PATH - 1);
				strncpy(paletteFilename, filename, len);
				paletteFilename[len - 4] = '.';
				paletteFilename[len - 3] = 'p';
				paletteFilename[len - 2] = 'a';
				paletteFilename[len - 1] = 'l';
				paletteFilename[len] = '\0';

				int paletteNum = portrait[portSelected].getPalNum();
				RGBTRIPLE prevPal[16];

				for (int i = 0; i < 16; ++i)
				{
					prevPal[i] = portraitPals[paletteNum][i];
				}

				auto paletteHandle = _open(paletteFilename, O_RDWR | O_BINARY);

				if (paletteHandle != -1)
				{
					BYTE temp[32];
					_read(paletteHandle, &temp, 32);

					BYTE* ptr = temp;
					for (int i = 0; i < 16; i++)
					{
						unsigned char c, d;
						c = *(ptr++);
						d = *(ptr++);
						portraitPals[paletteNum][i] = getRGBTRIPLE(c, d);
					}
					_close(paletteHandle);
				}

				auto portraitHandle = _open(filename, O_RDWR | O_BINARY);
				if (portraitHandle == -1)
				{
					for (int i = 0; i < 16; ++i)
					{
						portraitPals[paletteNum][i] = prevPal[i];
					}

					MessageBox(NULL, "Error opening file", ff3seTitle, MB_OK | MB_ICONERROR | MB_TASKMODAL);
					SetWindowLong(hdlg, DWL_MSGRESULT, 1);
					return 1;
				}

				BYTE temp[800];

				_read(portraitHandle, temp, 800);

				portrait[portSelected].load(temp);
				_close(portraitHandle);
				invalidatePics();
				return 0;
			}

			Bitmap newBMP;

			switch (newBMP.loadFile(((LPOFNOTIFY)lParam)->lpOFN->lpstrFile, 16, 40, 40))
			{
			case 0:
				portrait[portSelected].setImage(newBMP);
				invalidatePics();
				return 0;
				break;
			case 1:
				MessageBox(NULL, "Image has more than 16 colors", ff3seTitle, MB_OK | MB_ICONERROR | MB_TASKMODAL);
				break;
			case 2:
				MessageBox(NULL, "Image is larger than 40x40", ff3seTitle, MB_OK | MB_ICONERROR | MB_TASKMODAL);
				break;
			case 3:
				MessageBox(NULL, "Error opening file", ff3seTitle, MB_OK | MB_ICONERROR | MB_TASKMODAL);
				break;
			case 4:
				MessageBox(NULL, "File does not appear to be a valid image file", ff3seTitle, MB_OK | MB_ICONERROR | MB_TASKMODAL);
				break;
			}
			SetWindowLong(hdlg, DWL_MSGRESULT, 1);
			return 1;
			break;
		}
	}
	return 0;
}

/*************************************/
BOOL Tab3::tabProc(HWND hdlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	HWND hScroll;

	switch (message)
	{
	case WM_CLOSE:
		hTab = NULL;
		break;
	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDC_BUTTON_EXIT:
			SendMessage(hWnd, WM_CLOSE, 0, 0);
			return true;				case IDC_COMBO_PORTRAIT:
				if (HIWORD(wParam) == CBN_SELCHANGE)
				{
					newSelection();
				}
				return true;
				break;

				break;
			case IDC_BUTTON_PORTRAIT_BRUSH:
				if (HIWORD(wParam) == BN_CLICKED)
				{
					selectButton(0);
				}
				return true;
				break;
			case IDC_BUTTON_PORTRAIT_ERASE:
				if (HIWORD(wParam) == BN_CLICKED)
				{
					selectButton(1);
				}
				return true;
				break;
			case IDC_BUTTON_PORTRAIT_GRAB:
				if (HIWORD(wParam) == BN_CLICKED)
				{
					selectButton(2);
				}
				return true;
				break;
			case IDC_COMBO_PORTRAIT_PALETTE:
				if (HIWORD(wParam) == CBN_SELCHANGE)
				{
					changePalette();
				}
				return true;
				break;
			case IDC_BUTTON_PORTRAIT_CLEAR:
				clearImage();
				return true;
				break;
			case IDC_BUTTON_PORTRAIT_COPY:
				copyPortrait();
				return true;
				break;
			case IDC_BUTTON_PORTRAIT_IMPORT:
				import();
				return true;
				break;
			case IDC_CHECK_PORTRAIT_TRANSPARENT:
				if (HIWORD(wParam) == BN_CLICKED)
				{
					changeTransparency();
				}
				return true;
				break;
			case IDC_UPDOWN_PORTRAIT_MAGNIFICATION:
				invalidatePics();
				return true;
				break;
		}
		break;
	case WM_DRAWITEM:
		drawPaintButtons(wParam, lParam);
		break;
	case  WM_HSCROLL:
		hScroll = GetDlgItem(hTab, IDC_HSCROLL_PORTRAIT_EDIT);
		if ((HWND)lParam == hScroll)
		{
			handleMagScroll(LOWORD(wParam), (HWND)lParam, HIWORD(wParam));
			invalidatePics();
		}
		else
		{
			handleSliders(wParam, lParam);
		}
		return false;
		break;
	case WM_LBUTTONDOWN:
		handleMouseClick(LOWORD(lParam), HIWORD(lParam));
		return true;
		break;
	case WM_MOUSEMOVE:
		handleMouseMove(LOWORD(lParam), HIWORD(lParam), wParam & MK_LBUTTON);
		return true;
		break;
	case WM_PAINT:
		drawTab();
		return false;
		break;
	case WM_SETCURSOR:
		setCursor();
		return true;
		break;
	case WM_VSCROLL:
		hScroll = GetDlgItem(hTab, IDC_VSCROLL_PORTRAIT_EDIT);
		if ((HWND)lParam == hScroll)
		{
			handleMagScroll(LOWORD(wParam), (HWND)lParam, HIWORD(wParam));
		}
		invalidatePics();
		return false;
		break;
	}

	return false;
}

/*************************************/
BOOL CALLBACK tab3Proc(HWND hdlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	return tab3.tabProc(hdlg, message, wParam, lParam);
}

/*************************************/
BOOL CALLBACK copyPortraitProc(HWND hdlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	return tab3.copyImageProc(hdlg, message, wParam, lParam);
}

/*************************************/
UINT CALLBACK tab3ImportProc(HWND hdlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	return tab3.importProc(hdlg, message, wParam, lParam);
}

